package lejos.pc.tools;

import javax.swing.SwingUtilities;

public class ConsoleViewerSwingUI implements ConsoleViewerUI
{
	private static final int BUFSIZE = 4096;
	
	private final ConsoleViewerUI delegate;
	private char[] buf = new char[BUFSIZE];
	private int buflen;
	
	public ConsoleViewerSwingUI(ConsoleViewerUI delegate)
	{
		this.delegate = delegate;
	}
	
    /**
     * Messages generated by ConsoleViewComms show in the status Field
     */
    public void setStatus(final String s)
    {
    	SwingUtilities.invokeLater(new Runnable()
			{
				public void run()
				{
					delegate.setStatus(s);
				}
			});
    }
    
    public void connectedTo(final String name, final String address)
    {
    	SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				delegate.connectedTo(name, address);
			}
		});
    }

    /**
     * Append data to the log display
     */
    public void append(String data)
    {
		try
		{
	    	int len = data.length();
	    	for (int i=0; i<len;)
	    	{
	    		i += this.safeAppend(data, i);
	    	}	    	
		}
		catch (InterruptedException e)
		{
			throw new RuntimeException(e);
		}
    }
    
    private synchronized int safeAppend(String s, int off) throws InterruptedException
    {
    	int rem;
    	while (true)
    	{
    		rem = BUFSIZE - buflen;
	    	if (rem > 0)
	    		break;
	    	
	    	this.wait();
    	}
    	
    	int len = s.length() - off;
    	if (len <= 0)
    		return 0;
    	if (len > rem)
    		len = rem;
    	
    	int oldlen = buflen;
    	s.getChars(off, off+len, buf, buflen);
    	buflen += len;
    	
    	if (oldlen == 0)
    	{
    		SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						swingFlushBuffer();
					}
				});
    	}
    	
    	return len;
    }
    
    private synchronized void swingFlushBuffer()
    {
    	// method only invoked from swing thread
    	String s = new String(buf, 0, buflen);
    	delegate.append(s);
        buflen = 0;
        this.notifyAll();
    }

    public void updateLCD(final byte[] buffer)
    {
    	SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				delegate.updateLCD(buffer);
			}
		});
    }

	/**
	 * Log a progress message
	 */
	public void logMessage(final String msg) {
    	SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				delegate.logMessage(msg);
			}
		});
	}
}
